@@ -397,6 +397,48 @@
 #define NM_AVAILABLE_IN_1_48
 #endif
 
+#if NM_VERSION_MIN_REQUIRED >= NM_VERSION_1_50
+#define NM_DEPRECATED_IN_1_50        G_DEPRECATED
+#define NM_DEPRECATED_IN_1_50_FOR(f) G_DEPRECATED_FOR(f)
+#else
+#define NM_DEPRECATED_IN_1_50
+#define NM_DEPRECATED_IN_1_50_FOR(f)
+#endif
+
+#if NM_VERSION_MAX_ALLOWED < NM_VERSION_1_50
+#define NM_AVAILABLE_IN_1_50 G_UNAVAILABLE(1, 50)
+#else
+#define NM_AVAILABLE_IN_1_50
+#endif
+
+#if NM_VERSION_MIN_REQUIRED >= NM_VERSION_1_52
+#define NM_DEPRECATED_IN_1_52        G_DEPRECATED
+#define NM_DEPRECATED_IN_1_52_FOR(f) G_DEPRECATED_FOR(f)
+#else
+#define NM_DEPRECATED_IN_1_52
+#define NM_DEPRECATED_IN_1_52_FOR(f)
+#endif
+
+#if NM_VERSION_MAX_ALLOWED < NM_VERSION_1_52
+#define NM_AVAILABLE_IN_1_52 G_UNAVAILABLE(1, 52)
+#else
+#define NM_AVAILABLE_IN_1_52
+#endif
+
+#if NM_VERSION_MIN_REQUIRED >= NM_VERSION_1_54
+#define NM_DEPRECATED_IN_1_54        G_DEPRECATED
+#define NM_DEPRECATED_IN_1_54_FOR(f) G_DEPRECATED_FOR(f)
+#else
+#define NM_DEPRECATED_IN_1_54
+#define NM_DEPRECATED_IN_1_54_FOR(f)
+#endif
+
+#if NM_VERSION_MAX_ALLOWED < NM_VERSION_1_54
+#define NM_AVAILABLE_IN_1_54 G_UNAVAILABLE(1, 54)
+#else
+#define NM_AVAILABLE_IN_1_54
+#endif
+
 /*
  * Synchronous API for calling D-Bus in libnm is deprecated. See
  * https://networkmanager.dev/docs/libnm/latest/usage.html#sync-api
