@@ -500,7 +500,7 @@ show_nm_status(NmCli *nmc, const char *pretty_header_name, const char *print_fld
         fields_str = nmc->required_fields;
 
     if (!nmc_print_table(&nmc->nmc_config,
-                         (gpointer[]){nmc, NULL},
+                         (gpointer[]) {nmc, NULL},
                          NULL,
                          pretty_header_name ?: N_("NetworkManager status"),
                          (const NMMetaAbstractInfo *const *) metagen_general_status,
@@ -733,7 +733,7 @@ show_general_logging(NmCli *nmc)
         fields_str = nmc->required_fields;
 
     if (!nmc_print_table(&nmc->nmc_config,
-                         (gpointer const[]){&d, NULL},
+                         (gpointer const[]) {&d, NULL},
                          NULL,
                          _("NetworkManager logging"),
                          (const NMMetaAbstractInfo *const *) metagen_general_logging,
@@ -933,7 +933,7 @@ do_general_hostname(const NMCCommand *cmd, NmCli *nmc, int argc, const char *con
 
     hostname = *argv;
     if (next_arg(nmc, &argc, &argv, NULL) == 0)
-        nmc_print("Warning: ignoring extra garbage after '%s' hostname\n", hostname);
+        nmc_printerr("Warning: ignoring extra garbage after '%s' hostname\n", hostname);
 
     nmc->should_wait++;
     nm_client_save_hostname_async(nmc->client, hostname, NULL, save_hostname_cb, nmc);
@@ -1426,7 +1426,7 @@ ac_overview(NmCli *nmc, NMActiveConnection *ac)
     if (nm_active_connection_get_controller(ac)) {
         nm_str_buf_append_printf(&str,
                                  "%s %s, ",
-                                 _("master"),
+                                 _("controller"),
                                  nm_device_get_iface(nm_active_connection_get_controller(ac)));
     }
     if (nm_active_connection_get_vpn(ac))
